;******************************************************************************
;      * * * * * * *       T I C M A I N . A S M      * * * * * * * * 
;******************************************************************************
;       The Totally Insane Clock program for the DSP-93.
; Created       05-24-97        Moe Wheatley AE4JY
; Last Modified 10-28-97        Version 1.0
        .nolist
;
;                 Assembly Instructions
;invoking:  
;tasm -3225 -g0 -dRx -dGx -dXXX -dHxx -dYYYYYY ticmain.asm
;  where:       Rx = R1 or R2 for radio ports 1 or 2 (default=Port1)
;               Gx = G1,G2,G4,G8,G16,G32,G64   gain settings (default=G2)
;              XXX = GMT, EST, CST, MST, PST time zone setting (default=EST)
;              Hxx = H12 or H24 for 12 or 24 hour mode (default = H12)
;           YYYYYY = GONG or CUCKOO for hour chime (default is CUCKOO)   
;
;NOTE: The following source files are needed to assemble this code-
;  "ticmain.asm"
;  "ticinput.asm"
;  "tictime.asm"
;  "ticuart.asm"
;  "ticsound.asm"
;  "ticdata.tbl"
;
;The following include files from TAPR are needed to assemble this code-
;  "REGS.INC"
;  "PORTS.INC"
;  "MONITOR.INC"
;  "SERIAL.INC"
;
;                     L E D   U S E A G E
;  LED211 is seconds timing
;  LED212 is Seconds position locked
;  LED213 is Minute  position locked
;  LED214 is Minutes unit's digit locked( or oscillator compensated after lock)
;  LED215 is Minutes ten's digit locked
;  LED216 is Input is in AGC range
;  LED217 is Input level too high
;  LED218 is Power
;(after time acquisition, the LED's form a pendulum swing)
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1996.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact TAPR or the author for information on commercial licenses
; of this software.
; 
;------------------------------------------------------------------------------
;                     I N C L U D E   F I L E S
;------------------------------------------------------------------------------
#include  "REGS.INC"
#include  "PORTS.INC"
#include  "MONITOR.INC"
#include  "SERIAL.INC"
;        .list
;------------------------------------------------------------------------------
;                     M A C R O    D E F I N E S
;------------------------------------------------------------------------------
#define INC(xx) lac xx\ addk 1\ sacl xx
#define DEC(xy) lac xy\ subk 1\ sacl xy

;------------------------------------------------------------------------------
;            A R x    R e g i s t e r  U s e a g e
;------------------------------------------------------------------------------
;AR0 = general purpose temp pointer
;AR1 = general purpose temp pointer
;AR2 = general purpose temp pointer
;AR3 = Software Stack Pointer
;AR4 = Sample output que HEAD pointer (where data can be put for output)
;AR5 = Sample input que TAIL pointer (where data is taken for output and
;                                     input data placed during AIO irq )
;AR6 = Sample input que HEAD pointer ( where data can be taken out for input)
;AR7 = general purpose temp pointer
;------------------------------------------------------------------------------
;                     C O N S T A N T    E Q U A T E S
;------------------------------------------------------------------------------
TM_EN   .equ    008h    ;timer interrupt enable bit
RX_EN   .equ    010h    ;serial port recv enable bit
TX_EN   .equ    020h    ;serial port xmit enable bit
INT1_EN .equ    002h    ;external interrupt INT1 enable bit
CMEQ    .equ    0       ;cmpr instruction EQUALTO operator
CMLT    .equ    1       ;cmpr instruction LESSTHAN operator
CMGT    .equ    2       ;cmpr instruction GREATERTHAN operator
CMNE    .equ    3       ;cmpr instruction NOTEQUAL operator
;                               ;sample rate = 5MHz/(RA*RB)
;                               ;A/D filter 100 to 3700 Hz
CMDA    .equ    CMDA_VAL(25)    ;AIO load RA/TA = 17 for SCF @ 200 kHz
CMDB    .equ    CMDB_VAL(20)    ;AIO load RB/TB = 27 for 10000 samples/sec
;  AIO configuration: Filters out, Sync, 3 V full scale, sin x/x
CMDC    .equ    AIO_CONFIG|AIO_SYNC|AIO_GAIND61|AIO_GAIND70|AIO_INSSINX
;
MS5PERIOD       .equ    50000-1         ;Period for 5 Millisecond timer
                                        ; @ 10 MHZ   clock rate
;         UART Baud rate constants
BAUD2400        .equ    128             ;4.9152MHz/(2400*16)
BAUD4800        .equ    64              ;4.9152MHz/(4800*16)
BAUD9600        .equ    32              ;4.9152MHz/(9600*16)
BAUD19200       .equ    16              ;4.9152MHz/(19200*16)
BAUD38400       .equ    8               ;4.9152MHz/(38400*16)
BAUD76800       .equ    4               ;4.9152MHz/(76800*16)
;
;****   change BAUDRATE here for desired UART speed   *********
BAUDRATE        .equ    BAUD19200
;
RADIO_1         .equ    0               ;Radio port 1
RADIO_2         .equ    080h            ;Radio port 2
;
;       misc constants
BITSINBYTE      .equ    8               ;restating the obvious
CR              .equ    0Dh
LF              .equ    0Ah
;
;------------------------------------------------------------------------------
;         M E M O R Y   P A G E   P O I N T E R    E Q U A T E S
;------------------------------------------------------------------------------
USER_ADR        .equ    USER_PAGE<<7
;
;------------------------------------------------------------------------------
;         I N T E R N A L   M E M O R Y    E Q U A T E S
;------------------------------------------------------------------------------
;B0 range is 200h to 2FFh       used for filter coefficients
;       (when addressed as program memory, 0FF00h to 0FFFF)
B0DADDR         .equ    0200h
B0PADDR         .equ    0FF00h
;
AA1FIR_LENGTH   .equ    25
AA2FIR_LENGTH   .equ    63
BP1FIR_LENGTH   .equ    63
BP2FIR_LENGTH   .equ    63
LP1FIR_LENGTH   .equ    11
LP2FIR_LENGTH   .equ    11
BP1FIRCOEF      .equ    B0DADDR         ;start of coefficient tables
AA1FIRCOEF      .equ    BP1FIRCOEF+BP1FIR_LENGTH
AA2FIRCOEF      .equ    AA1FIRCOEF+AA1FIR_LENGTH
LP1FIRCOEF      .equ    AA2FIRCOEF+AA2FIR_LENGTH
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;B1 range is 300h to 3FFh       used for filter delay lines
BP1Fdelay       .equ    300h
AA1Fdelay       .equ    BP1Fdelay+BP1FIR_LENGTH+1
AA2Fdelay       .equ    AA1Fdelay+AA1FIR_LENGTH+1
BP2Fdelay       .equ    AA2Fdelay+AA2FIR_LENGTH+1
LP1Fdelay       .equ    BP2Fdelay+BP1FIR_LENGTH+1
LP2Fdelay       .equ    LP1Fdelay+LP1FIR_LENGTH+1
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;B2 range is 060h to 07Fh       used for software stack and fast variables
;
ToutBuf         .equ    060h
;
EndStack        .equ    070h
Stack           .equ    07Fh    ;Stack grows down from 7F to 70;
;------------------------------------------------------------------------------
;               E X T E R N A L   M E M O R Y    E Q U A T E S
;------------------------------------------------------------------------------
;Variables in external memory addressed by direct mode (400h to 47Fh)
Temp0           .equ    000h
Temp1           .equ    001h
Temp2           .equ    002h
Temp3           .equ    003h
Port6mirror     .equ    004h
Port7mirror     .equ    005h
PortBmirror     .equ    006h
Xmit_Phz_Acc    .equ    007h
OutBuf          .equ    008h
TestOut         .equ    009h
;
Timer1          .equ    00Ah
Timer2          .equ    00Bh
TIC_Mask_timer  .equ    00Ch
AGC_Sum_l       .equ    00Dh
AGC_Sum_h       .equ    00Eh
AGC_Ave         .equ    00Fh
TIC_Sum_l       .equ    010h
TIC_Sum_h       .equ    011h
TIC_Ave         .equ    012h
DeltaNew        .equ    013h
DeltaOld        .equ    014h
OldInput        .equ    015h
Max_meas        .equ    016h
Level_1000      .equ    017h
Deci_count1     .equ    018h
Deci_count2     .equ    019h
Decout_1        .equ    01Ah
Decout_2        .equ    01Bh
Peak1000        .equ    01Ch
Late_Sum_l      .equ    01Dh
Late_Sum_h      .equ    01Eh
Deci_mod5       .equ    01Fh
Early_Sum_l     .equ    020h
Early_Sum_h     .equ    021h
Sec_timer       .equ    022h
Level_100       .equ    023h
TIC_quality     .equ    024h
Sec_adj         .equ    025h
Sec_count       .equ    026h
Min_count       .equ    027h
Hr_count        .equ    028h
Sec_pos         .equ    029h
Msec_5flag      .equ    02Ah
N1K_Sum_l       .equ    02Bh
N1K_Sum_h       .equ    02Ch
S1K_Sum_l       .equ    02Dh
S1K_Sum_h       .equ    02Eh
N100_Sum_l      .equ    02Fh
N100_Sum_h      .equ    030h
S100A_Sum_l     .equ    031h
S100A_Sum_h     .equ    032h
S100B_Sum_l     .equ    033h
S100B_Sum_h     .equ    034h
Trial_count     .equ    035h
Lock_timer      .equ    036h
bit_limit       .equ    037h
bit_total_lim   .equ    038h
Min_units       .equ    039h
Clk_error       .equ    03Ah
Clk_adj         .equ    03Bh
Clk_adj_timer   .equ    03Ch
Meas_interval   .equ    03Dh
LED_ptr         .equ    03Eh
LED_time        .equ    03Fh
;                          
AGC_limit       .equ    040h
AD_Limit        .equ    041h
Hardware        .equ    042h
Perftimer       .equ    043h
Output_scale    .equ    044h
BitData         .equ    045h
Min_status      .equ    046h
Local_offset    .equ    047h
;
tbl_pos         .equ    050h
Timer2reload    .equ    051h
Sound_position  .equ    052h
Sound_ptr       .equ    053h
Prev_sec        .equ    054h
Prev_min        .equ    055h
Prev_hr         .equ    056h
Gong_count      .equ    057h
Local_hr        .equ    058h
;
Hz1000_Vector   .equ    070h
Sound_Vector    .equ    071h

; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;               B I T    V A R I A B L E S
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Flags           .equ    07Fh    ;bit variables
RADIO_PORT      .equ    1<<0    ;selects radio port(0=port1, 1=port2)
T_RADIO_PORT    .equ    0       ;test bit
PTT1_LOGIC      .equ    1<<1    ;selects PTT port1 logic sense(0=NEG, 1=POS)
T_PTT1_LOGIC    .equ    1       ;test bit
PTT2_LOGIC      .equ    1<<2    ;selects PTT port2 logic sense(0=NEG, 1=POS)
T_PTT2_LOGIC    .equ    2       ;test bit
PEAK_DET        .equ    1<<3    ;Peak Detect latch(1=A/D peak detected)
T_PEAK_DET      .equ    3       ;test bit
CW_ACTIVE       .equ    1<<4    ;CW ID is active
T_CW_ACTIVE     .equ    4       ;test bit
POS_LOCKED      .equ    1<<5    ;Seconds Position is locked
T_POS_LOCKED    .equ    5       ;test bit
SEC_LOCKED      .equ    1<<6    ;Seconds are locked
T_SEC_LOCKED    .equ    6       ;test bit
MIN_LOCKED      .equ    1<<7    ;Minutes units are locked
T_MIN_LOCKED    .equ    7       ;test bit
MIN10_LOCKED    .equ    1<<8    ;Minutes 10's are locked
T_MIN10_LOCKED  .equ    8       ;test bit
DAY_SAVINGS     .equ    1<<9    ;Dayloght savings is in affect
T_DAY_SAVINGS   .equ    9       ;test bit
TIME_OK         .equ    1<<10   ;Time has been locked
T_TIME_OK       .equ    10      ;test bit
CLK_FAST        .equ    1<<11   ;DSP93 Clock is running fast
T_CLK_FAST      .equ    11      ;test bit
CLK_MEAS        .equ    1<<12   ;Measuring DSP93 Clock
T_CLK_MEAS      .equ    12      ;test bit
SOUND_ACTIVE    .equ    1<<13   ;Sound output is active
T_SOUND_ACTIVE  .equ    13      ;test bit
CLK_CORRECTED   .equ    1<<14   ;Clk is adjusted for accuracy
T_CLK_CORRECTED .equ    14      ;test bit
PM_TIME         .equ    1<<15   ;PM time flag
T_PM_TIME       .equ    15      ;test bit
;
;Variables in external memory addressed by indirect mode (500h to 7FFFh)
SAMPQUESIZE     .equ    8
INQUESIZE       .equ    50
OUTQUESIZE      .equ    50
MINARRAYSIZE    .equ    60
MIN10ARRAYSIZE  .equ    5
HRARRAYSIZE     .equ    8
DSTARRAYSIZE    .equ    1
ASCIIARRAYSIZE  .equ    13
FMGENSIZE       .equ    8
;                
Sample_Que:     .org    0500h
TICin_Que:      .org    $+SAMPQUESIZE
TICout_Que:     .org    $+INQUESIZE
Min_Array:      .org    $+OUTQUESIZE
Min10_Array:    .org    $+MINARRAYSIZE
Hr_Array:       .org    $+MIN10ARRAYSIZE
DST_Array:      .org    $+HRARRAYSIZE
ASCII_Time:     .org    $+DSTARRAYSIZE
ASCII_Status:   .org    $+ASCIIARRAYSIZE
FMgen1:         .org    $+ASCIIARRAYSIZE
FMgen2:         .org    $+FMGENSIZE
FMgen3:         .org    $+FMGENSIZE
;
;==============================================================================
;     ==========   I N T E R R U P T    V E C T O R S   ==========
;==============================================================================
        .org    1000h
        b       TimerIntService ;branch to timer interrupt service (TINT)
        b       RxIntService    ;SIO receive interrupt service routine
        b       TxIntService    ;SIO transmit interrupt service routine
        b       Trap            ;branch to trap routine (TRAP)
        b       Start           ;branch to program start (RESET Vector)
Trap:
        eint
        ret
;
;+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
;         E X T E R N A L     M O D U L E     I N C L U D E S 
;+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
#include  "TICdata.tbl"        ;Include constant tables
#include  "TICinput.asm"
#include  "TICsound.asm"
#include  "TICtime.asm"
#include  "TICuart.asm"
;
;==============================================================================
;     ==========   P R O G R A M   E N T R Y   P O I N T   ==========
;==============================================================================
Start:  
        dint                    ;disable interrupts
        rptk    7-1             ;free up hardware stack
        pop
        call    Init_System     ;Initialize the system
;
;==============================================================================
;       ==========    M A I N   P R O G R A M   L O O P     ==========
;==============================================================================
;       Poor man's multitasking Round Robin calling of all the tasks.
MainLoop:
;$$$$
;        call    FREQ_UP_1off
;        lac     Perftimer      ;this test code was used to measure
;        subk    1              ;the processing time through all the
;        sacl    Perftimer      ;service routines.
;        bnz     ML_1
;        lalk    500
;        sacl    Perftimer
;        call    FREQ_UP_1on        
;ML_1:
;$$$$
        call    Service_Ain     ;service input samples from A/D Que
;
        call    Service_Sound   ;service output sound samples to D/A
;
        call    Service_TICuart ;service TIC data from UART
;
        call    Service_TICtime ;service TIC time calculations
;
        b       MainLoop        ;loop round and round
;
;==============================================================================
;       ==========    I / O   S U B R O U T I N E S     ==========
;==============================================================================
;
;................     PORT 6 I/O routines     .................................
RD_Ion: lac     Port6mirror
        ork     RDI             ;RD_I bit
        b       Port6_Write
RD_Ioff:
        lac     Port6mirror
        andk    ~RDI            ;RD_I bit
        b       Port6_Write
RD_Clkon:
        lac     Port6mirror
        ork     RCLKI           ;RCLKI bit
        b       Port6_Write
RD_Clkoff:
        lac     Port6mirror
        andk    ~RCLKI          ;RCLKI bit
        b       Port6_Write
LED5Ton:
        bit     Flags,T_MIN10_LOCKED
        bbnz    LED5on
        ret
LED5on: lac     Port6mirror
        ork     LED_205         ;LED 205 bit
        b       Port6_Write
LED5off:
        lac     Port6mirror
        andk    LED_205_OFF
        b       Port6_Write
LED6on: lac     Port6mirror
        ork     LED_206         ;LED 206 bit
        b       Port6_Write
LED6off:
        lac     Port6mirror
        andk    LED_206_OFF
        b       Port6_Write
LED7on: lac     Port6mirror
        ork     LED_207         ;LED 207 bit
        b       Port6_Write
LED7off:
        lac     Port6mirror
        andk    LED_207_OFF
Port6_Write:
        sacl    Port6mirror
        out     Port6mirror,TNC_OUTPUT
        ret
;................     PORT 7 I/O routines     .................................
AIO_Reset:
        lac     Port7mirror
        ork     AIO_RESET               ;reset D15 to disconnect AIO (DX&DR)
        b       Port7_Write
AIO_Run:
        lac     Port7mirror
        andk    ~AIO_RESET              ;bring AIO out of reset
        b       Port7_Write
AIO_Enable:
        lac     Port7mirror
        ork     AIO_ENABLE
        b       Port7_Write
AIO_Disable:
        lac     Port7mirror
        andk    ~AIO_ENABLE
        b       Port7_Write
Sel_Input:                              ;Temp0 contains selection--
        lac     Port7mirror             ;(see PORTS.INC for definitions)
        andk    0FFF8h                  ;clear select bits
        or      Temp0
        b       Port7_Write
Sel_Gain:                               ;Temp0 contains selection--
        lac     Port7mirror             ;(see PORTS.INC for definitions)
        andk    0FFC7h                  ;clear gain bits
        or      Temp0
        b       Port7_Write
LED1on: lac     Port7mirror
        ork     LED_201                 ;LED 201 bit
        b       Port7_Write
LED1off:
        lac     Port7mirror
        andk    LED_201_OFF
        b       Port7_Write
LED2Ton:
        bit     Flags,T_POS_LOCKED
        bbnz    LED2on
        ret
LED2on: lac     Port7mirror
        ork     LED_202                 ;LED 202 bit
        b       Port7_Write
LED2off:
        lac     Port7mirror
        andk    LED_202_OFF
Port7_Write:
        sacl    Port7mirror
        out     Port7mirror,RADIO_GAIN
        ret
;
;................     PORT B I/O routines     .................................
PTT1_on:
        lac     PortBmirror             ;PTT radio port 1
        bit     Flags,T_PTT1_LOGIC      ;check logic sense
        bbz     PTT1_0                  ;br if NEG logic sense
        andk    ~PTT_R1
        b       PortB_Write
PTT1_0: ork     PTT_R1
        b       PortB_Write
PTT1_off:
        lac     PortBmirror             ;PTT radio port 1
        bit     Flags,T_PTT1_LOGIC      ;check logic sense
        bbz     PTT1_1                  ;br if NEG logic sense
        ork     PTT_R1
        b       PortB_Write
PTT1_1: andk    ~PTT_R1
        b       PortB_Write
;
PTT2_on:
        lac     PortBmirror             ;PTT radio port 2
        bit     Flags,T_PTT2_LOGIC      ;check logic sense
        bbz     PTT2_0                  ;br if NEG logic sense
        andk    ~PTT_R2
        b       PortB_Write
PTT2_0: ork     PTT_R2
        b       PortB_Write
PTT2_off:
        lac     PortBmirror             ;PTT radio port 2
        bit     Flags,T_PTT2_LOGIC      ;check logic sense
        bbz     PTT2_1                  ;br if NEG logic sense
        ork     PTT_R2
        b       PortB_Write
PTT2_1: andk    ~PTT_R2
        b       PortB_Write
;
LED3Ton:
        bit     Flags,T_SEC_LOCKED
        bbnz    LED3on
        ret
LED3on: lac     PortBmirror
        ork     LED_203         ;LED 203 bit
        b       PortB_Write
LED3off:
        lac     PortBmirror
        andk    LED_203_OFF
        b       PortB_Write
LED4Ton:
        bit     Flags,T_MIN_LOCKED
        bbnz    LED4on
        ret
LED4T2on:
        bit     Flags,T_CLK_CORRECTED
        bbnz    LED4on
        ret
LED4on: lac     PortBmirror
        ork     LED_204         ;LED 204 bit
        b       PortB_Write
LED4off:
        lac     PortBmirror
        andk    LED_204_OFF
        b       PortB_Write
FREQ_UP_1on:
        lac     PortBmirror
        ork     FREQ_UP_R1      ;freq up port 1
        b       PortB_Write
FREQ_UP_1off:
        lac     PortBmirror
        andk    ~FREQ_UP_R1
        b       PortB_Write
FREQ_UP_2on:
        lac     PortBmirror
        ork     FREQ_UP_R2      ;freq up port 2
        b       PortB_Write
FREQ_UP_2off:
        lac     PortBmirror
        andk    ~FREQ_UP_R2
        b       PortB_Write
FREQ_DN_1on:
        lac     PortBmirror
        ork     FREQ_DN_R1      ;freq down port 1
        b       PortB_Write
FREQ_DN_1off:
        lac     PortBmirror
        andk    ~FREQ_DN_R1
        b       PortB_Write
FREQ_DN_2on:
        lac     PortBmirror
        ork     FREQ_DN_R2      ;freq down port 2
        b       PortB_Write
FREQ_DN_2off:
        lac     PortBmirror
        andk    ~FREQ_DN_R2
PortB_Write:
        sacl    PortBmirror
        out     PortBmirror,RADIO_CTRL
        ret
;
;==============================================================================
;            ==========    D e l a y x ( AR0 )     ==========
;==============================================================================
;This routine is called with AR0 containing the delay interval in 5 mSec units.
;The routine waits until the Timer1 value decrements from the specified count
;to zero.  The Timer IRQ decrements the Timer1 value.
;
Delayx: larp	AR0
        lrlk    AR0,Timer1+USER_ADR
        sacl    *
Dlx_1:  zals    *
        bnz     Dlx_1
        ret
;
;=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
;  ======  I N T E R R U P T   S E R V I C E   R O U T I N E S  =========
;=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

;==============================================================================
;         ======    R x I n t S e r v i c e ( )      =======
;==============================================================================
;   Since the A/D and D/A are running at the same sample rate(10000 Hz),  only
;the receive interrupt is used for both.
;void RxIntService(){
;   Save_Context();
;   DXR = Sample_Que[AR6];              // write D/A from Sample_Que
;   Sample_Que[AR6++] = DRR;            // read A/D into Sample_Que
;   if( AR6>Sample_Que+SAMPQUESIZE-1 )  // deal with wrap around
;      AR6 = Sample_Que;
;   Restore_Context();
;}
RxIntService:                   ;SIO receive interrupt service routine
        larp    AR3             ;save context on software stack
        mar     *-              ;stackpointer--
        sst1    *-              ;push save status reg 1
        sst     *-              ;push save status reg 0
        sar     AR0,*,AR6       ;save AR0 and point to next sample in
;                                  Sample_Que
        ldpk    BASE_PAGE
        lar     AR0,*           ;AR0=next xmit sample
;$$$$$
;     ldpk    USER_PAGE          ;useful for outputing test data
;     lar     AR0,TestOut
;     ldpk    BASE_PAGE
;$$$$$
        sar     AR0,DXR         ;send to D/A converter for output
;
        lar     AR0,DRR         ;AR0=next A/D sample
        sar     AR0,*+          ;put in Sample_Que and inc AR6 head pointer
        lrlk    AR0,Sample_Que+SAMPQUESIZE-1    ;check for wrap around
        cmpr    CMGT            ;TC = (AR6>AR0)
        bbz     Rx1             ;br if no wrap
        lrlk    AR6,Sample_Que  ;else wrap it around
;
Rx1:    larp    AR3             ;restore context
        lar     AR0,*+          ;restore AR0
        lst     *+              ;pop status 0
        lst1    *+              ;pop status 1
        eint
        ret
;..............................................................................
;==============================================================================
;                 T x I n t S e r v i c e ( )
;==============================================================================
TxIntService:                   ;SIO transmit interrupt service routine
        lac     ToutBuf         ;used only during initialization
        sacl    DXR
        eint
        ret
;..............................................................................
;==============================================================================
;                 T i m e r I n t S e r v i c e ( )
;==============================================================================
;void TimerIntService(){
;   Save_Context();
;   if( Timer1 )                // decrement Timer1 down to zero
;      Timer1--;
;   if( Timer2 )                // decrement Timer2 down to zero
;      Timer2--;
;   if( TIC_Mask_timer )        // decrement TIC_Mask_timer down to zero
;      TIC_Mask_timer--;
;   Restore_Context();
;}
;
TimerIntService:
        larp    AR3             ;save context on software stack
        mar     *-              ;stackpointer--
        sst1    *-              ;push save status reg 1
        sst     *-              ;push save status reg 0
        sar     AR0,*           ;save AR0
;
        ldpk    USER_PAGE
        larp    AR0             ;ARP = AR0
        lar     AR0,Timer1      ;AR0 = Timer1
        banz    TI1,*-          ;br if( AR0-- != 0);
        b       TI2             ;else Timer1 is Zero so keep it zero
TI1:    sar     AR0,Timer1      ;Timer1--;
TI2:
        lar     AR0,Timer2      ;AR0 = Timer2
        banz    TI3,*-          ;br if( AR0-- != 0);
        b       TI4             ;else Timer2 is Zero so keep it zero
TI3:    sar     AR0,Timer2      ;Timer2--;
TI4:
        lar     AR0,TIC_Mask_timer       ;AR0 = TIC_Mask_timer
        banz    TI5,*-          ;br if( AR0-- != 0);
        b       TI6             ;else TIC_Mask_timer is Zero so keep it zero
TI5:    sar     AR0,TIC_Mask_timer       ;TIC_Mask_timer--;
TI6:
        lark    AR0,1           ;set flag to signal 5 msec irq happened
        sar     AR0,Msec_5flag
;
        larp    AR3             ;restore context
        lar     AR0,*+          ;restore AR0
        lst     *+              ;pop status 0
        lst1    *+              ;pop status 1
        eint
        ret
;..............................................................................
;==============================================================================
;         =========    I n i t _ S y s t e m ( )    ==========
;==============================================================================
;This routine initializes the various hardware and software variables
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
Init_System:
        ldpk    BASE_PAGE       ;data mem page 0 (internal)
        ssxm                    ;set sign extension mode
        sovm                    ;saturate on overflow
        spm     1               ;set P shift for Q15
        fort    0               ;AIO = 16 bit mode
        sfsm                    ;use external frame sync signal for receive
        rtxm                    ;use external frame sync signal for xmit
        cnfp                    ;B0 as program memory
;
        lrlk    AR3,Stack       ;init software stack        
;
        lalk    MS5PERIOD       ;Init 5 Millisecond Timer
        sacl    PRD             ;set up 5 millisecond timer
        lalk    TM_EN           ;make Timer interrupt active
        sacl    IMR             ;set INT mask
        eint                    ;enable interrupts
;
        ldpk    USER_PAGE       ;DEBUG stuff
        lalk    0h
        sacl    DEBUGPC         ;useage flag for debug routine( see TAPR docs.)
        lalk    0460h           ;point to first memory location used in page 8.
        sacl    DEBUGML
;
        zac
        sacl    Port6mirror
        sacl    Port7mirror
        sacl    PortBmirror
        sacl    Timer1
        sacl    Timer2
        sacl    TIC_Mask_timer

;................ Initialize A/D Converter chip........................
        call    AIO_Disable
        call    AIO_Reset
;
        call    AIO_Enable
        call    AIO_Run
        lalk    20
        call    Delayx          ;delay 20 x .005 = 100 ms
;
;................ initialize AIO  .............................................
        ldpk    BASE_PAGE       ;data mem page 0 (internal)
        lalk    TX_EN           ;mask SIO TX interrupt active
        sacl    IMR             ;set INT mask for tx irq's
        zac
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        eint                    ;enable TX interrupt
        idle                    ;wait for TX interrupt
        lalk    AIO_CONFIG      ;load AIO command flag
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    CMDC            ;load AIO config command
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    AIO_CONFIG      ;load AIO command flag
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    CMDB            ;load TB/RB AIO command next
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    AIO_CONFIG      ;load AIO command flag
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    CMDA            ;load TA/RA AIO command last
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
;
        dint                    ;disable interrupts
        lalk    RX_EN|TM_EN     ;make SIO RX and timer interrupt active
        sacl    IMR             ;store mask in IMR
        ldpk    USER_PAGE       ;data mem page 8 (external)
;
        zac
        sacl    Flags
        call    Init_TICtime    ;initialize external modules
        call    Init_TICuart
        call    Init_Ain
        call    Init_Aout
;
        eint                    ;enable interrupts
;
;. . . . . . . . . . U A R T   S E T U P . . . . . . . . . . . . . . . . .
;change rate to BAUDRATE Bits per second, 8 bits, no parity, one stop bit
        lalk    UART_SEL_LCR    ;select Line Control register
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp1,UART_READ ;read current Line Control values
        lac     Temp1
        ork     UART_LCR_DLAB
        sacl    Temp1
        out     Temp1,UART_WRITE
        lalk    UART_SEL_DL_MSB
        sacl    Temp0
        out     Temp0,UART_CTRL ;select MSB of buad rate divisor
        lalk    (BAUDRATE >> 8)
        sacl    Temp0
        out     Temp0,UART_WRITE
;
        lalk    UART_SEL_DL_LSB
        sacl    Temp0
        out     Temp0,UART_CTRL ;select LSB of buad rate divisor
        lalk    (BAUDRATE & 0FFh)
        sacl    Temp0
        out     Temp0,UART_WRITE
;
        lalk    UART_SEL_LCR    ;select Line Control register
        sacl    Temp0
        out     Temp0,UART_CTRL
;
        lac     Temp1           ;deselect baud rate divisor register
        andk    ~UART_LCR_DLAB
        sacl    Temp1
        out     Temp1,UART_WRITE
;
        lalk    UART_SEL_MCR    ;select Modem Control register
        sacl    Temp0
        out     Temp0,UART_CTRL
	lack    UART_MCR_RTS|UART_MCR_DTR
	sacl    Temp0
        out     Temp0,UART_WRITE
;
        ret
;==============================================================================
;      ============  E N D  o f   T I C M A I N . A S M  ===========
;==============================================================================
        .end
