;******************************************************************************
;                         T I C T I M E . A S M
;******************************************************************************
;   This module processes and decodes the time data from the 100Hz and 1000Hz
; energy bins.  Time of day clock maintenance and internal oscillator clk
;adjustment calculations are also here.
;
S100A_THRESH    .equ    2*12616 ;3dB S+N/N  S = 2 * ( 2*2*25690/32768)*34 =105
                                ;       N = (59+46) = 105
S100B_THRESH    .equ    2*7168  ;3dB S+N/N  S = 2 * ( 2*2*14562/32768)*60 = 105
S1K_THRESH      .equ    5183    ;5dB S+N/N  3.16=1/(2*5183/32768)
BIT_LIMIT_L     .equ    4       ;Initial individual bit energy limit
BIT_LIMIT_H     .equ    6       ;Higher individual bit energy limit
BIT_TOTAL_LIM_3 .equ    18      ;Total bit energy threshold for 3 bits
BIT_TOTAL_LIM_4 .equ    24      ;Total bit energy threshold for 4 bits
BIT_TOTAL_LIM_6 .equ    36      ;Total bit energy threshold for 6 bits
LOCK_TIMEOUT1   .equ    30      ;minute units lock limit in minutes
LOCK_TIMEOUT2   .equ    60      ;10 minute units lock limit in minutes
LOCK_TIMEOUT3   .equ    90      ;Hrs lock limit in minutes
MEAS_INC        .equ    375     ; 6.25*60 = number of 5msec adj per minute
                                ;  == ( .005/(.0002*4) ) * 60
MEAS_MAX        .equ    (30000-MEAS_INC)
MEAS_15MIN      .equ    (15*MEAS_INC)
MEAS_30MIN      .equ    (30*MEAS_INC)

;==============================================================================
;      ========          I n i t _ T I C t i m e ( )     ==========
;==============================================================================
Init_TICtime:                   ;initialize variables
        lalk    LED_STATUS_TBL
        sacl    LED_ptr
        lack    10
        sacl    LED_time
;
        lalk    65535
        sacl    Clk_adj
        sacl    Clk_adj_timer
        lalk    MEAS_15MIN
        sacl    Max_meas;
;
        zac
        sacl    Trial_count
        sacl    Msec_5flag
        sacl    Sec_count
        sacl    Min_count
        sacl    Hr_count
        sacl    N1K_Sum_l  
        sacl    N1K_Sum_h  
        sacl    S1K_Sum_l  
        sacl    S1K_Sum_h  
        sacl    N100_Sum_l 
        sacl    N100_Sum_h 
        sacl    S100A_Sum_l
        sacl    S100A_Sum_h
        sacl    S100B_Sum_l
        sacl    S100B_Sum_h
;
        lack    1
        sacl    Sec_pos
;
        call    Start_resync
        ret
;.............................................................................
;========================================================================
;                 S e r v i c e _ T I C t i m e ( )
;========================================================================
; Serviced every time 5 Msec timer interval.
;void Service_TICtime()
:{
;   if( Msec_5flag ){
;      Msec_5flag = FALSE;
;      Sec_pos--;
;      Service_1000Hz();
;      Service_100Hz();
;      switch( Sec_pos ){
;         case 0:
;            if( --Clk_adj_timer == 0 ){
;               Clk_adj_timer = Clk_adj;
;               if( Flags.CLK_FAST )
;                  Sec_pos = 201;
;               else
;                  Sec_pos = 199;
;            }else
;               Sec_pos = 200;
;            LED1on();
;            break;
;         case 1:
;            Service_Sec();
;            break;
;         case 2:
;            DATASERVICE[ Sec_count ](); // indirect call using table
;            break;
;         default:                      //LED sequencer state machine
;            if( LED_time == Sec_pos ){
;               ACC = LED_ptr;          // point to present LED state
;               LED_ptr = *(ACC++);     // get next state
;               LED_time = *(ACC++);    // get next trigger time
;               Temp0 = *ACC;
;               *Temp0();               // indirect call action routine
;            }
;            break;
;      }
;   }
;}
;
Service_TICtime:
        lac     Msec_5flag
        bz      ST_exit
        zac
        sacl    Msec_5flag
;
        lac     Sec_pos                 ; Sec_pos--;
        subk    1
        bnz     ST_2
;
        DEC(Clk_adj_timer)              ; if( --Clk_adj_timer == 0 ){
        bnz     ST_1                    ;    Clk_adj_timer = Clk_adj;
        lac     Clk_adj                 ;    if( Flags.CLK_FAST )
        sacl    Clk_adj_timer           ;       Sec_pos = 201;
        lack    201                     ;    else
        bit     Flags,T_CLK_FAST        ;       Sec_pos = 199;
        bbnz    ST_2                    ; }else
        lack    199                     ;    Sec_pos = 200;
        b       ST_2
ST_1:   lack    200
ST_2:   sacl    Sec_pos
;
        call    Service_1000Hz          ; chk for minute tone
        call    Service_100Hz           ; get 100Hz bit data value
;
        lack    2
        sub     Sec_pos
        bnz     ST_3
        larp    AR7                     ; DATASERVICE[ Sec_count ]();
        lac     Sec_count               ; ARP = Data Array pointer
        adlk    DATASERVICE             ; indirect call using call table
        tblr    Temp0
        lac     Temp0
        cala
        b       ST_exit
ST_3:   lack    1
        sub     Sec_pos
        bnz     ST_4
        call    Service_Sec             ;called to inc time counters
        b       ST_exit
ST_4:
        lac     LED_time        ; if( LED_time == Sec_pos ){
        sub     Sec_pos         ;    ACC = LED_ptr;
        bnz     ST_exit         ;    LED_ptr = *(ACC++);
        lac     LED_ptr         ;    LED_time = *(ACC++);
        tblr    LED_ptr         ;    Temp0 = *ACC;
        addk    1               ;    *Temp0();
        tblr    LED_time
        addk    1
        tblr    Temp0
        lac     Temp0
        cala
ST_exit:
        ret

;========================================================================
;                     S r v _ M/H x ( )
;========================================================================
; Routines to service the minutes and hour data bits
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;void Srv_M1()
:{
;   Integrate_Array( Min_Array+1 );
;}
;void Srv_M2()
:{
;   Integrate_Array( Min_Array+2 );
;}
;void Srv_M4()
:{
;   Integrate_Array( Min_Array+3 );
;}
;void Srv_M8()
:{
;   Integrate_Array( Min_Array+4 );
;}
;void Srv_M10()
:{
;   Integrate_bit( M10_Array+1 );
;}
;void Srv_M20()
:{
;   Integrate_bit( M10_Array+2 );
;}
;void Srv_M40()
:{
;   Integrate_bit( M10_Array+3 );
;   Calc_Bits( Min10_Array );
;}
;void Srv_H1()
:{
;   Integrate_bit( Hr_Array+1 );
;}
;void Srv_H2()
:{
;   Integrate_bit( Hr_Array+2 );
;}
;void Srv_H4()
:{
;   Integrate_bit( Hr_Array+3 );
;}
;void Srv_H10()
:{
;   Integrate_bit( Hr10_Array+4 );
;}
;void Srv_H20()
:{
;   Integrate_bit( Hr10_Array+5 );
;   Calc_Bits( Hr_Array );
;}
;
;void Integrate_Array( AR0 ){
;   for( i=AR0; i<AR0+60; i+=6 )
;      Min_Array[i] += BitData;
;}
;void Integrate_bit( AR0 ){
;   Min_Array[AR0] += BitData;
;}
;..........................................................................
;
Nop:    ret                             ;called in inactive second time slots
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M1:
        lrlk    AR0,Min_Array+1
        call    Integrate_Array
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M2:
        lrlk    AR0,Min_Array+2
        call    Integrate_Array
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M4:
        lrlk    AR0,Min_Array+3
        call    Integrate_Array
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M8:
        lrlk    AR0,Min_Array+4
        call    Integrate_Array
        lrlk    AR0,Min_Array
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+6
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+12
        lack    4
        sacl    Temp3
        call    Calc_bits
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M10:
        lrlk    AR0,Min10_Array+1
        call    Integrate_bit
        lrlk    AR0,Min_Array+18
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+24
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+30
        lack    4
        sacl    Temp3
        call    Calc_bits
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M20:
        lrlk    AR0,Min10_Array+2
        call    Integrate_bit
        lrlk    AR0,Min_Array+36
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+42
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+48
        lack    4
        sacl    Temp3
        call    Calc_bits
        lrlk    AR0,Min_Array+54
        lack    4
        sacl    Temp3
        call    Calc_bits
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Srv_M40:
        lrlk    AR0,Min10_Array+3
        call    Integrate_bit
        lrlk    AR0,Min10_Array
        lack    3
        sacl    Temp3
        call    Calc_bits
        lac     Temp0
        lrlk    AR0,Min10_Array
        sacl    *
        ret
Srv_H1:
        lrlk    AR0,Hr_Array+1
        call    Integrate_bit
        ret
Srv_H2:
        lrlk    AR0,Hr_Array+2
        call    Integrate_bit
        ret
Srv_H4:
        lrlk    AR0,Hr_Array+3
        call    Integrate_bit
        ret
Srv_H8:
        lrlk    AR0,Hr_Array+4
        call    Integrate_bit
        ret
Srv_H10:
        lrlk    AR0,Hr_Array+5
        call    Integrate_bit
        ret
Srv_H20:
        lrlk    AR0,Hr_Array+6
        call    Integrate_bit
        lrlk    AR0,Hr_Array
        lack    6
        sacl    Temp3
        call    Calc_bits
        lac     Temp0
        lrlk    AR0,Hr_Array
        sacl    *
        ret

Srv_DST:
        lrlk    AR0,DST_Array
        call    Integrate_bit
        ret

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Integrate_Array:
        larp    AR0             ;ARP = AR0
        lrlk    AR1,10-1        ;AR1 = loop counter
Int_A1: lac     *
        add     BitData
        sacl    *               ;[AR0] += BitData;
        adrk    6               ;AR0 += 6;
        larp    AR1             ;ARP = AR1
        banz    Int_A1,*-,AR0
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Integrate_bit:
        larp    AR0             ;ARP = AR0
        lac     *
        add     BitData
        sacl    *               ;[AR0] += BitData;
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;========================================================================
;                     I n c _ M i n _ R o w s ( )
;========================================================================
; Routine to increment the minutes unit data arrays
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Inc_Min_Rows( ){
;   AR0 = Min_Array;
;   AR1 = 10-1;
;   do{
;      Temp1 = MOD10_NEXT_TBL;
;      Temp3 = 4;
;      Inc_bits();
;      AR1--;
;   }while( AR1 !=0 );
;}
Inc_Min_Rows:
        lrlk    AR0,Min_Array
        lrlk    AR1,10-1
        larp    AR0
Inc_M1:
        lalk    MOD10_NEXT_TBL
        sacl    Temp1
        lack    4
        sacl    Temp3
        call    Inc_bits
        larp    AR1
        banz    Inc_M1,*-,AR0
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;========================================================================
;                     I n c _ M i n 1 0 ( )
;========================================================================
; Routine to increment the minutes tens data
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Inc_Min10( ){
;   if( Flags.MIN_LOCKED && (Min_units == 9) ){
;      Temp1 = MOD6_NEXT_TBL;
;      AR0 = Min10_Array
;      Temp3 = 3;
;      Inc_bits();
;   }
;}
Inc_Min10:
        bit     Flags,T_MIN_LOCKED
        bbz     Inc_Mx
        lac     Min_units
        subk    9
        bnz     Inc_Mx
        larp    AR0
        lrlk    AR0,Min10_Array
        lalk    MOD6_NEXT_TBL
        sacl    Temp1
        lack    3
        sacl    Temp3
        call    Inc_bits
Inc_Mx: ret
;..........................................................................
;========================================================================
;                     I n c _ H r s ( )
;========================================================================
; Routine to increment the hours data
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Inc_Hrs( ){
;   if( Flags.MIN10_LOCKED && (Min_count == 0) ){
;      Temp1 = MOD24_NEXT_TBL;
;      AR0 = Hr_Array
;      Temp3 = 6;
;      Inc_bits();
;   }
;}
Inc_Hrs:
        bit     Flags,T_MIN10_LOCKED
        bbz     Inc_Hrx
        lac     Min_count
        bnz     Inc_Hrx
        larp    AR0
        lrlk    AR0,Hr_Array
        lalk    MOD24_NEXT_TBL
        sacl    Temp1
        lack    6
        sacl    Temp3
        call    Inc_bits
Inc_Hrx: ret
;..........................................................................
;========================================================================
;                     C a l c _ b i t s ( )
;========================================================================
; Routine to calculate value of the Temp3 bits in array AR0 and also the
; bit energy totals.
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;                 Bit array format
;     0       1      2      3      4      5      6        n+2
;  --------------------------------------------------------------
; | value | Bit0 | Bit1 | Bit2 | Bit3 | Bit4 | Bitn | energy sum |
;  --------------------------------------------------------------
;parameters:
;     AR0 = pointer to bit energy array
;     Temp3 = Number of bits in array
;returns:
;     Temp0 = value of bits
;
;Calc_bits( AR0, Temp3 ){
;   Temp0 = 0;
;   Temp1 = 0;
;   Temp2 = FALSE;
;
;   Temp1 += abs( AR0[1] );
;   if( abs( AR0[1] ) < bit_limit )
;      Temp2 = TRUE;
;   if( AR0[1] > 0 )
;      Temp0 |= 1;
;   if( --Temp3 ){
;      Temp1 += abs( AR0[2] );
;      if( abs( AR0[2] ) < bit_limit )
;         Temp2 = TRUE;
;      if( AR0[2] > 0 )
;         Temp0 |= 2;
;      if( --Temp3 ){
;         Temp1 += abs( AR0[3] );
;         if( abs( AR0[3] ) < bit_limit )
;            Temp2 = TRUE;
;         if( AR0[3] > 0 )
;            Temp0 |= 4;
;         if( --Temp3 ){
;            Temp1 += abs( AR0[4] );
;            if( abs( AR0[4] ) < bit_limit )
;               Temp2 = TRUE;
;            if( AR0[4] > 0 )
;               Temp0 |= 8;
;            if( --Temp3 ){
;               Temp1 += abs( AR0[5] );
;               if( abs( AR0[5] ) < bit_limit )
;                  Temp2 = TRUE;
;               if( AR0[5] > 0 )
;                  Temp0 |= 16;
;               if( --Temp3 ){
;                  Temp1 += abs( AR0[6] );
;                  if( abs( AR0[6] ) < bit_limit )
;                     Temp2 = TRUE;
;                  if( AR0[6] > 0 )
;                     Temp0 |= 32;
;               }
;            }
;         }
;      }
;   }
;   if( Temp2 == FALSE ){
;      *(++AR0) = Temp1;
;   }else{
;      *(++AR0) = -Temp1;
;   }
;}. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Calc_bits:
        larp    AR0             ;ARP = AR0
;
        zac
        sacl    Temp0           ;   Temp0 = 0;
        sacl    Temp1           ;   Temp1 = 0;
        sacl    Temp2           ;   Temp2 = FALSE;
        mar     *+
;
        lac     *
        abs
        add     Temp1
        sacl    Temp1           ;   Temp1 += abs( AR0[1] );
        sub     bit_limit
        bgez    Calc_b1         ;   if( abs( AR0[1] ) < bit_limit )
        sacl    Temp2           ;      Temp2 = TRUE // flag low energy bit
Calc_b1:
        lac     *+              ;if( AR0[1] > 0 )
        blez    Calc_b2         ;  Temp0 |= 1;
        lac     Temp0
        ork     1
        sacl    Temp0
Calc_b2:
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Calc_bx         ;if( --Temp3 ){
        lac     *
        abs
        add     Temp1
        sacl    Temp1           ;   Temp1 += abs( AR0[2] );
        sub     bit_limit
        bgez    Calc_b3         ;   if( abs( AR0[2] ) < bit_limit )
        sacl    Temp2           ;      Temp2 = TRUE // flag low energy bit
Calc_b3:
        lac     *+              ;   if( AR0[2] > 0 )
        blez    Calc_b4         ;      Temp0 |= 2;
        lac     Temp0
        ork     2
        sacl    Temp0
Calc_b4:
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Calc_bx         ;if( --Temp3 ){
        lac     *
        abs
        add     Temp1
        sacl    Temp1           ;   Temp1 += abs( AR0[3] );
        sub     bit_limit
        bgez    Calc_b5         ;   if( abs( AR0[3] ) < bit_limit )
        sacl    Temp2           ;      Temp2 = TRUE // flag low energy bit
Calc_b5:
        lac     *+              ;   if( AR0[3] > 0 )
        blez    Calc_b6         ;      Temp0 |= 4;
        lac     Temp0
        ork     4
        sacl    Temp0
Calc_b6:
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Calc_bx         ;if( --Temp3 ){
        lac     *
        abs
        add     Temp1
        sacl    Temp1           ;   Temp1 += abs( AR0[4] );
        sub     bit_limit
        bgez    Calc_b7         ;   if( abs( AR0[4] ) < bit_limit )
        sacl    Temp2           ;      Temp2 = TRUE // flag low energy bit
Calc_b7:
        lac     *+              ;   if( AR0[4] > 0 )
        blez    Calc_b8         ;      Temp0 |= 8;
        lac     Temp0
        ork     8
        sacl    Temp0
Calc_b8:
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Calc_bx         ;if( --Temp3 ){
        lac     *
        abs
        add     Temp1
        sacl    Temp1           ;   Temp1 += abs( AR0[5] );
        sub     bit_limit
        bgez    Calc_b9         ;   if( abs( AR0[5] ) < bit_limit )
        sacl    Temp2           ;      Temp2 = TRUE // flag low energy bit
Calc_b9:
        lac     *+              ;   if( AR0[5] > 0 )
        blez    Calc_b10        ;      Temp0 |= 16;
        lac     Temp0
        ork     16
        sacl    Temp0
Calc_b10:
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Calc_bx         ;if( --Temp3 ){
        lac     *
        abs
        add     Temp1
        sacl    Temp1           ;   Temp1 += abs( AR0[6] );
        sub     bit_limit
        bgez    Calc_b11        ;   if( abs( AR0[6] ) < bit_limit )
        sacl    Temp2           ;      Temp2 = TRUE // flag low energy bit
Calc_b11:
        lac     *+              ;   if( AR0[6] > 0 )
        blez    Calc_bx         ;      Temp0 |= 32;
        lac     Temp0
        ork     32
        sacl    Temp0
Calc_bx:
        lac     Temp2           ;   if( Temp2 == FALSE ){
        bz      Calc_b12        ;      *(++AR0) = Temp1;
        lac     Temp1           ;   }else{
        neg
        sacl    Temp1           ;      *(++AR0) = -Temp1
Calc_b12:
        lac     Temp1           ;   }
        sacl    *
        ret
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;========================================================================
;                     I n c _ b i t s ( )
;========================================================================
; Routine to increment the Temp3 bits in array AR0, modulus table Temp1,
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;                 Bit array format
;     0       1      2      3      4      5      n        n+2
;  --------------------------------------------------------------
; | value | Bit0 | Bit1 | Bit2 | Bit3 | Bit4 | Bitn | energy sum |
;  --------------------------------------------------------------
;parameters:
;     AR0 = pointer to bit energy array
;     Temp1 = pointer to MODULUS table
;     Temp3 = Number of bits in array
;
;Inc_bits( AR0, Temp1, Temp3){
;   Temp2 = AR0[0];
;   Temp1 = Temp1[ Temp2 ];
;   AR0[0] = Temp1;
;   Temp0 = Temp1 ^ Temp2;
;
;   if( Temp0&BIT0 )
;      AR0[1] = -AR0[1];
;   }
;   if( --Temp3 ){
;      if( Temp0&BIT1 )
;         AR0[2] = -AR0[2];
;      if( --Temp3 ){
;         if( Temp0&BIT2 )
;            AR0[3] = -AR0[3];
;         if( --Temp3 ){
;            if( Temp0&BIT3 )
;               AR0[4] = -AR0[4];
;            if( --Temp3 ){
;               if( Temp0&BIT4 )
;                  AR0[5] = -AR0[5];
;               if( --Temp3 ){
;                  if( Temp0&BIT5 )
;                     AR0[6] = -AR0[6];
;               }
;            }
;         }
;      }
;   }
;   AR0++;
;}
Inc_bits:
        larp    AR0
;
        lac     *               ;   Temp2 = AR0[0]; // current value
        sacl    Temp2
        add     Temp1
        tblr    Temp1           ;   Temp1 = Temp1[ Temp2 ]; // next value
        lac     Temp1
        sacl    *+              ;   AR0[0] = Temp1;
        xor     Temp2
        sacl    Temp0           ;   Temp0 = Temp1 ^ Temp2;
;
        lac     *
        bit     Temp0,0         ;   if( Temp0&BIT0 )
        bbz     Ib1
        neg                     ;      AR0[1] = -AR0[1];
Ib1:    sacl    *+
;
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Inc_bx          ;   if( --Temp3 ){
        lac     *
        bit     Temp0,1         ;   if( Temp0&BIT1 )
        bbz     Ib2
        neg                     ;      AR0[2] = -AR0[2];
Ib2:    sacl    *+
;
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Inc_bx          ;   if( --Temp3 ){
        lac     *
        bit     Temp0,2         ;   if( Temp0&BIT2 )
        bbz     Ib3
        neg                     ;      AR0[3] = -AR0[3];
Ib3:    sacl    *+
;
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Inc_bx          ;   if( --Temp3 ){
        lac     *
        bit     Temp0,3         ;   if( Temp0&BIT3 )
        bbz     Ib4
        neg                     ;      AR0[4] = -AR0[4];
Ib4:    sacl    *+
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Inc_bx          ;   if( --Temp3 ){
        lac     *
        bit     Temp0,4         ;   if( Temp0&BIT4 )
        bbz     Ib5
        neg                     ;      AR0[5] = -AR0[5];
Ib5:    sacl    *+
        lac     Temp3
        subk    1
        sacl    Temp3
        bz      Inc_bx          ;   if( --Temp3 ){
        lac     *
        bit     Temp0,5         ;   if( Temp0&BIT5 )
        bbz     Ib6
        neg                     ;      AR0[6] = -AR0[6];
Ib6:    sacl    *+
Inc_bx: 
        mar     *+              ;point to beginning of next array row
        ret
;
;========================================================================
;                     C h k _ M i n ( )
;========================================================================
; Routine to check minute energy for valid data.
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Chk_Min( ){
;   if( Lock_timer!=0 || Flags.SEC_LOCKED)
;      Lock_timer++;
;   if( !Flags.MIN10_LOCKED ){     // if minutes tens not locked
;      AR0 = Min_Array-6;          //BCD position ptr
;      AR1 = Min_Array+5-6;        //Total energy per phase ptr
;      AR2=10-1;
;      Temp0 = 0;
;      Min_status = 0;
;      do{                         // calculate minute units value
;         AR1 += 6;
;         Temp1 = [AR1];
;         Temp2 = abs[AR1];
;         AR0 += 6;
;         if( Temp2 > Min_status ){
;            Min_status = Temp2;   // save new max value for status indicator
;         if( Temp1 > Temp0 ){
;            Temp0 = Temp1;        // save new max value
;            Min_units = [AR0];    // save which phase is now max energy
;         }
;      }while(  AR2-- > 0 );
;      if( !Flags.MIN_LOCKED){        // if minutes units not locked
;         if( Lock_timer > LOCK_TIMEOUT1 ){
;            Start_resync();
;         }else{
;            if( Temp0 >= bit_total_lim ){
;               Flags.MIN_LOCKED = TRUE;
;               if( Lock_timer >= Min_units ){
;                  Clear_Min10_data();
;               }
;            }
;         }
;       }else{                              // here if Min units are locked
;         if( Lock_timer > LOCK_TIMEOUT2 ){
;            Start_resync();
;         }else{
;            if( M10_Array[4] >= bit_total_lim ){
;               Min_count = Min_units + 10*Min10_Array[0];
;               Flags.MIN10_LOCKED = TRUE;
;               if( Lock_timer >= Min_count )
;                  Clear_Hr_data();
;            }
;         }
;      }
;   }
;}
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Chk_Min:
                                        ;if( Lock_timer!=0 || Flags.SEC_LOCKED)
        bit     Flags,T_SEC_LOCKED
        bbnz    Chk_M0
        lac     Lock_timer
        bz      Chk_M1
Chk_M0: INC(Lock_timer)
;
Chk_M1: bit     Flags,T_MIN10_LOCKED    ;if( !Flags.MIN10_LOCKED ){
        bbnz    Chk_Mx
;
        lrlk    AR0,Min_Array-6         ;BCD position ptr
        lrlk    AR1,Min_Array+5-6       ;energy value ptr
        lrlk    AR2,10-1                ;loop ptr
        zac
        sacl    Min_status              ;max value holder for status
        sacl    Temp0                   ;max value holder
        larp    AR1
Chk_M2:
        adrk    6                       ; AR1 += 6
        lac     *,0,AR0                 ;acc = [AR1], AR=AR0
        adrk    6                       ; AR0 += 6
        sacl    Temp1                   ;Temp1 = abs[AR1]
        abs
        sacl    Temp2                   ;Temp2 = [AR1]
        sub     Min_status
        blez    Chk_M3
        lac     Temp2
        sacl    Min_status
Chk_M3:
        lac     Temp1
        sub     Temp0
        blez    Chk_M4
        lac     Temp1
        sacl    Temp0
        lac     *
        sacl    Min_units
Chk_M4:
        larp    AR2
        banz    Chk_M2,*-,AR1
;
        bit     Flags,T_MIN_LOCKED
        bbnz    Chk_M6
        lac     Lock_timer              ;if( Lock_timer > LOCK_TIMEOUT1 ){
        subk    LOCK_TIMEOUT1
        blez    Chk_M5
        call    Start_resync            ;   Start_resync();
        b       Chk_Mx
Chk_M5:
        lac     Temp0
        sub     bit_total_lim
        blz     Chk_Mx          ;   if( Temp0 >= bit_total_lim ){
        lac     Flags           ;      Flags.MIN_LOCKED = TRUE;
        ork     MIN_LOCKED
        sacl    Flags
        lack    BIT_TOTAL_LIM_3
        sacl    bit_total_lim
        call    Calc_Min
        lac     Lock_timer      ;      if( Lock_timer >= Min_units )
        sub     Min_units       ;         Clear_Min10_data();
        blz     Chk_Mx
        call    Clear_Min10_data
        b       Chk_Mx
;
Chk_M6:                         ; here if units are locked
        lac     Lock_timer      ;if( Lock_timer > LOCK_TIMEOUT2 ){
        subk    LOCK_TIMEOUT2
        blez    Chk_M7
        call    Start_resync    ;   Start_resync();
        b       Chk_Mx
Chk_M7:                         ;}else{
        larp    AR0
        lrlk    AR0,Min10_Array+4 ;if( M10_Array[4] >= bit_total_lim ){
        lac     *
        sub     bit_total_lim
        blz     Chk_Mx
;                                 
        lrlk    AR0,Min10_Array   
        lt      *
        lac     Min_units
        mpyk    5               ;actually *10 since mpyk shifts left 1
        apac
        sacl    Min_count       ; Min_count = Min_units + 10*Min10_Array[0];
        call    Calc_Min
        lac     Flags
        ork     MIN10_LOCKED
        sacl    Flags           ; Flags.MIN10_LOCKED = TRUE;
        lack    BIT_TOTAL_LIM_6
        sacl    bit_total_lim
        lac     Lock_timer      ; if( Lock_timer >= Min_count )
        sub     Min_count       ;    Clear_Hr_data();
        blz     Chk_Mx
        call    Clear_Hr_data
Chk_Mx:
        ret

;..........................................................................
;========================================================================
;                        C h k _ H r s ( )
;========================================================================
; Routine to check hour energy for valid data.
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Chk_Hrs( ){
;   if( Flags.MIN10_LOCKED){  // if minutes are all locked
;      if( Lock_timer > LOCK_TIMEOUT3 ){
;         Start_resync();
;      }else{
;         if( Hr_Array[7] >= bit_total_lim ){
;            Hr_count = Hr_Array[0] & 0x0F;
;            Hr_count += (Hr_Array[0] >> 4) * 10;
;            if( !Flags.TIME_OK){          // if time not locked
;               Prev_hr = Hr_count;        // keep gongs from sounding
;               Prev_min = Min_count;      // immediately
;               LED_ptr = LED_SEQ_TBL;
;            }
;            if( DST_Array[0] >= 0 )
;               Flags.DAY_SAVINGS = TRUE;
;            Calc_Hr();
;            Start_resync();
;            bit_limit = BIT_LIMIT_H;   // set higher thresholds
;            Flags.TIME_OK = TRUE;
;         }
;      }
;   }
;}
;
Chk_Hrs:
        bit     Flags,T_MIN10_LOCKED
        bbz     Chk_Hx
;
        lac     Lock_timer      ;if( Lock_timer > LOCK_TIMEOUT3 ){
        subk    LOCK_TIMEOUT3
        blez    Chk_H1
        call    Start_resync    ;   Start_resync();
        b       Chk_Hx
Chk_H1:                         ;}else{
        larp    AR0
        lrlk    AR0,Hr_Array+7 ;if( Hr_Array[7] >= bit_total_lim ){
        lac     *
        sub     bit_total_lim
        blz     Chk_Hx
        lrlk    AR0,Hr_Array    ; Hr_units = Hr_Array[0];
        lac     *
        andk    0Fh
        sacl    Hr_count        ; Hr_count = Hr_Array[0] & 0x0F;
        lac     *
        sfr
        sfr
        sfr
        sfr
        sacl    Temp0
        lt      Temp0
        lac     Hr_count
        mpyk    5               ;actually *10 since mpyk shifts left 1
        apac
        sacl    Hr_count        ; Hr_count += (Hr_Array[0] >> 4) * 10;
;
        bit     Flags,T_TIME_OK ;if( !Flags.TIME_OK){    
        bbnz    Chk_H3          ;   Prev_hr = Hr_count;  
        lac     Hr_count        ;   Prev_min = Min_count;
        sacl    Prev_hr         ;   LED_ptr = LED_SEQ_TBL;
        lac     Min_count       ;}
        sacl    Prev_min
        lalk    LED_SEQ_TBL
        sacl    LED_ptr
Chk_H3:
        lac     Flags
        andk    ~DAY_SAVINGS
        sacl    Flags           ; Flags.DAY_SAVINGS = FALSE;
        larp    AR0
        lrlk    AR0,DST_Array
        lac     *
        blez    Chk_H2          ;if( DST_Array[0] >= 0 )
        lac     Flags
        ork     DAY_SAVINGS
        sacl    Flags           ;   Flags.DAY_SAVINGS = TRUE;
Chk_H2:
        call    Calc_Hr
        call    Start_resync    ;   Start_resync();
        lack    BIT_LIMIT_H
        sacl    bit_limit
        lac     Flags
        ork     TIME_OK
        sacl    Flags           ; Flags.TIME_OK = TRUE;
Chk_Hx:
        ret
;========================================================================
;                 C a l c _ c l k _ e r r o r ( )
;========================================================================
;void Calc_clk_error( void)
;{      
;   if( Flags.CLK_MEAS ){
;      Meas_interval += MEAS_INC;       // 5*60 = #5msec per minute
;      if( Meas_interval >= MEAS_MAX ){ // is at max accuracy so
;         Flags.CLK_MEAS = FALSE;       // quit measuring
;      }
;      else{
;         if( Meas_interval > Max_meas ){ // make sure is greater than last
;            Max_meas = Meas_interval;
;            if( Clk_error > 0 )
;               Flags.CLK_FAST = TRUE;
;            else
;               Flags.CLK_FAST = FALSE;
;            Clk_adj = Meas_interval/abs(Clk_error);
;            Clk_adj_timer = Clk_adj;
;            if( Meas_interval >= MEAS_30MIN ){
;               Flags.CLK_CORRECTED = TRUE;
;            }
;         }
;      }
;   }
;}
Calc_clk_error:
        bit     Flags,T_CLK_MEAS
        bbz     Cce_exit                ;if not in measure mode
        lac     Meas_interval
        adlk    MEAS_INC                ; Meas_interval += MEAS_INC;
        sacl    Meas_interval
        adlk    -(MEAS_MAX)
        blz     Cce_1                   ; if( Meas_interval >= MEAS_MAX )
        lac     Flags
        andk    ~CLK_MEAS               ; Flags.CLK_MEAS = FALSE;
        sacl    Flags
        b       Cce_exit
Cce_1:
        lac     Meas_interval
        sub     Max_meas
        blez    Cce_exit        ; if( Meas_interval > Max_meas )
        lac     Meas_interval   ;     Max_meas = Meas_interval;
        sacl    Max_meas
;
        lac     Clk_error
        blz     Cce_2
        lac     Flags
        ork     CLK_FAST        ; Flags.CLK_FAST = TRUE;
        b       Cce_3
Cce_2:  lac     Flags
        andk    ~CLK_FAST       ; Flags.CLK_FAST = FALSE;
Cce_3:  sacl    Flags
        lac     Clk_error
        bnz     Cce_4
        lack    1               ;don't divide by zero
Cce_4:
        abs
        sacl    Temp0
        lac     Meas_interval
        sacl    Temp1
        call    Divide          ; Clk_adj = Meas_interval/abs(Clk_error);
        lac     Temp0
        sacl    Clk_adj
        sacl    Clk_adj_timer   ; Clk_adj_timer = Clk_adj;
;
        lac     Meas_interval
        adlk    -(MEAS_30MIN)
        blz     Cce_exit        ; if( Meas_interval >= MEAS_30MIN )
        lac     Flags
        ork     CLK_CORRECTED
        sacl    Flags           ;Flags.CLK_CORRECTED = TRUE;
Cce_exit:
        ret

;========================================================================
;                 S e r v i c e _ S e c ( )
;========================================================================
; Called at the zero position of Sec_pos.
;void Service_Sec()
:{
;   Calc_Sec();
;   if( ++Trial_count > 59 )
;      Trial_count = 0;
;   if( ++Sec_count > 59 ){
;      Sec_count = 0;
;      Calc_Min();
;      if( ++Min_count > 59 ){
;         Min_count = 0;
;         Calc_Hr();
;         if( ++Hr_count > 23 ){
;            Hr_count = 0;
;         }
;      }
;   }
;   Calc_Sec();
;}
;
Service_Sec:
;
        INC(Trial_count)
        subk    59
        blez    SS_1
        zac
        sacl    Trial_count
SS_1:
        INC(Sec_count)
        subk    59
        blez    SS_8
        zac
        sacl    Sec_count
;
        INC(Min_count)
        subk    59
        blez    SS_7
        zac
        sacl    Min_count
;
        INC(Hr_count)
        subk    23
        blez    SS_6
        zac
        sacl    Hr_count
SS_6:
        call    Calc_Hr
SS_7:   call    Calc_Min
SS_8:   call    Calc_Sec
        ret

;========================================================================
;                 S e r v i c e _ 1 0 0 0 H z ( )
;========================================================================
;void Service_1000Hz()
:{
;   if( Sec_pos == 1 ){
;      if( Flags.POS_LOCKED ){
;         if( S1K_Sum*S1K_THRESH >= N1K_Sum ){
;            if( Trial_count == 0 ){
;               Flags.SEC_LOCKED = TRUE;
;               Sec_count = 0;
;               if( !Flags.CLK_MEAS ){
;                  Meas_interval = 0;
;                  Clk_error = 0;
;                  Flags.CLK_MEAS = TRUE;
;               }
;            }else{
;               if( Flags.SEC_LOCKED == FALSE ){
;                  Trial_count = 0;
;               }
;               Flags.SEC_LOCKED == FALSE
;            }
;         }
;      }else{
;         Flags.SEC_LOCKED == FALSE
;         Flags.CLK_MEAS = FALSE;
;      }
;      S1K_Sum = 0;
;      N1K_Sum = 0;
;   }else{
;      if( Sec_pos > 181 || ( Sec_pos > 1 && Sec_pos < 20 ) )
;         N1K_Sum += Level_1000;
;      else
;         S1K_Sum += Level_1000;
;   }
;}
;
Service_1000Hz:
        lack    1
        sub     Sec_pos
        bnz     S1000_1
;
        bit     Flags,T_POS_LOCKED
        bbz     S1000_5         ;if seconds position not locked
;
        lt      S1K_Sum_h
        lalk    S1K_THRESH
        sacl    Temp0
        mpy     Temp0           ; P = S1K_Sum_h * S1000_THRESH
        pac
        subh    N1K_Sum_h
        blz     S1000_3         ;br if( S1000_THRESH*S1K_Sum < N1K_Sum )
;                               ;here if signal is 5 dB over noise
        lac     Trial_count
        bnz     S1000_4         ; br if wrong position
        lac     Flags
        ork     SEC_LOCKED
        sacl    Flags           ; Flags.Sec_Locked = TRUE;
        zac
        sacl    Sec_count
        bit     Flags,T_CLK_MEAS
        bbnz    S1000_3         ;br if already in measure mode
        zac        
        sacl    Meas_interval
        sacl    Clk_error
        lac     Flags
        ork     CLK_MEAS
        sacl    Flags           ; Flags.POS_LOCKED = TRUE;
        b       S1000_3
S1000_4:
        bit     Flags,T_SEC_LOCKED
        bbnz    S1000_5         ;if Seconds locked
        zac
        sacl    Trial_count
S1000_5:
        lac     Flags           ; Flags.CLK_MEAS = FALSE;
        andk    ~(SEC_LOCKED|CLK_MEAS)
        sacl    Flags           ; Flags.Sec_Locked = FALSE;
S1000_3:
        zac
        sacl    N1K_Sum_l
        sacl    N1K_Sum_h
        sacl    S1K_Sum_l
        sacl    S1K_Sum_h
        b       S1000_exit
S1000_1:
        lack    181
        sub     Sec_pos
        blz     S1000_2
        lack    20
        sub     Sec_pos
        bgz     S1000_2
;
        zals    S1K_Sum_l       ;integrate S1K sum over 160 samples
        addh    S1K_Sum_h       ;from 180 down to 20
        add     Level_1000,8
        sach    S1K_Sum_h
        sacl    S1K_Sum_l
        b       S1000_exit
S1000_2:
        zals    N1K_Sum_l       ;integrate N1K sum over 38 samples
        addh    N1K_Sum_h       ;from 200 down to 181 and 19 down to 1
        add     Level_1000,10   ;integrate 4 times faster than S1K_Sum
        sach    N1K_Sum_h
        sacl    N1K_Sum_l
S1000_exit:
        ret
;========================================================================
;                 S e r v i c e _ 1 0 0 H z ( )
;========================================================================
;void Service_100Hz()
:{
;   if( Sec_pos == 3 ){
;      BitData = 0;
;      if( Flags.SEC_LOCKED ){
;         if( (S100A_Sum*S100A_THRESH) >= N100_Sum ){
;            if( (S100B_Sum*S100B_THRESH) >= N100_Sum )
;               BitData = 1;
;            else
;               BitData = -1;
;         }
;      }
;      S100A_Sum = 0;
;      S100B_Sum = 0;
;      N100_Sum = 0;
;   }else{
;      if( Sec_pos > 154 )
;         N100_Sum += Level_100;        // 1024*46 over noise sample interval
;      }else{
;         if( Sec_pos > 120 ){
;            S100A_Sum += Level_100;    // 1024*34 over S100A sample interval
;         }else{
;            if( Sec_pos > 60 ){
;               S100B_Sum += Level_100; // 1024*60 over S100B sample interval
;            }else{
;               N100_Sum += Level_100;  // 1024*60 over noise sample interval
;            }
;         }
;      }
;   }
;}
;
Service_100Hz:
        lack    3
        sub     Sec_pos
        bnz     S100_3
        zac
        sacl    BitData
        bit     Flags,T_SEC_LOCKED
        bbz     S100_2          ;br if seconds not locked
;
        lt      S100A_Sum_h
        lalk    S100A_THRESH
        sacl    Temp0
        mpy     Temp0           ; P = S100A_Sum_h * S100A_THRESH
        pac
        subh    N100_Sum_h
        blz     S100_2          ;br if( S100A_THRESH*S100A_Sum < N100_Sum )
;
        lt      S100B_Sum_h
        lalk    S100B_THRESH
        sacl    Temp0
        mpy     Temp0           ; P = S100B_Sum_h * S100B_THRESH
        pac
        subh    N100_Sum_h
        blz     S100_1          ;br if( S100B_THRESH*S100B_Sum < N100_Sum )
        lack    1
        sacl    BitData         ;valid signal "1"
        b       S100_2
S100_1:                         ;if valid signal "0"
        lalk    -1
        sacl    BitData         ;valid signal "0"
S100_2: zac
        sacl    N100_Sum_l 
        sacl    N100_Sum_h 
        sacl    S100A_Sum_l
        sacl    S100A_Sum_h
        sacl    S100B_Sum_l
        sacl    S100B_Sum_h
        b       S100_exit
S100_3:
        lack    154
        sub     Sec_pos
        bgez    S100_5          ;br if Sec_pos <= 154
S100_4:
        zals    N100_Sum_l      ;integrate N100 sum over 46+59 samples
        addh    N100_Sum_h      ;from 200 down to 154 && 60 down to 1
        add     Level_100,9     ;add in Level_100 * 512
        sach    N100_Sum_h
        sacl    N100_Sum_l
        b       S100_exit
;
S100_5:                         ;here if Secpos <= 154
        lack    120
        sub     Sec_pos
        bgez    S100_6          ;br if Sec_pos <= 120
        zals    S100A_Sum_l     ;integrate S100A sum over 34 samples
        addh    S100A_Sum_h     ;from 154 down to 120
        add     Level_100,10    ;add in Level_100 * 1024
        sach    S100A_Sum_h
        sacl    S100A_Sum_l
        b       S100_exit
S100_6:                         ;here if Secpos <= 120
        lack    60

        sub     Sec_pos
        bgez    S100_4          ;integrate N100 sum over 59 to 1
        zals    S100B_Sum_l     ;integrate S100B sum over 60 samples
        addh    S100B_Sum_h     ;from 120 down to 60
        add     Level_100,10    ;add in Level_100 * 1024
        sach    S100B_Sum_h
        sacl    S100B_Sum_l
S100_exit:
        ret

;========================================================================
;                 C l e a r _ M i n _ d a t a ( )
;========================================================================
Clear_Min_data:
        larp    AR0             ;ARP = Data_array
        lrlk    AR0,Min_Array
        zac
        rptk    MINARRAYSIZE-1 ;clear Minute date array
        sacl    *+
;
        lrlk    AR0,Min_Array
        lrlk    AR1,10-1
        zac                     ;init phase array with sequence 0-9
CMd_1:  sacl    *
        adrk    6
        larp    AR1
        addk    1
        banz    CMd_1,*-,AR0
        ret
;
Clear_Min10_data:
        larp    AR0             ;ARP = Data_array
        lrlk    AR0,Min10_Array
        zac
        rptk    MIN10ARRAYSIZE-1 ;clear Minute10 date array
        sacl    *+
        ret
;
;========================================================================
;                 C l e a r _ H r _ d a t a ( )
;========================================================================
Clear_Hr_data:
        larp    AR0             ;ARP = Data_array
        lrlk    AR0,Hr_Array
        zac
        rptk    HRARRAYSIZE-1   ;clear Hour data array
        sacl    *+
        ret
;========================================================================
;                 S t a r t _ r e s y n c ( )
;========================================================================
Start_resync:
        call    Clear_Min_data
        call    Clear_Min10_data
        call    Clear_Hr_data
        lrlk    AR0,DST_Array
        zac
        sacl    *               ;DST_Array[0] = 0;
        sacl    Lock_timer
        sacl    Min_status              ;max value holder for min status
        lack    BIT_LIMIT_L
        sacl    bit_limit
        lack    BIT_TOTAL_LIM_4
        sacl    bit_total_lim
        lac     Flags
        andk    ~(SEC_LOCKED|MIN_LOCKED|MIN10_LOCKED)
        sacl    Flags
        bit     Flags,T_CLK_MEAS
        bbnz     Sr_1            ;if in measure mode don't resync position
        zac
        sacl    TIC_quality
        lac     Flags
        andk    ~POS_LOCKED
        sacl    Flags
        lalk    Search_1000
        sacl    Hz1000_Vector
Sr_1:
        ret
;==============================================================================
;                 =======   D i v i d e ( Temp1/Temp0 )     =======
;==============================================================================
;   This routine divides Temp1 by Temp0.
;if( Temp0 == 0 )
;   Temp0 = 32767;
;else
;   Temp0 = Temp1/Temp0;
;------------------------------------------------------------------------------
Divide:
        lac     Temp0
        bnz     Div_1
        lalk    32767
        sacl    Temp0
        ret        
Div_1:  lac     Temp1
        rptk    15
        subc    Temp0
        sacl    Temp0
        ret
;==============================================================================
;     ============== E N D   o f  T I C T I M E  . A S M ==============
;==============================================================================
